@@ -2,7 +2,7 @@
 # PODNAME: App::Prove::Plugin::TestRail
 
 package App::Prove::Plugin::TestRail;
-$App::Prove::Plugin::TestRail::VERSION = '0.026';
+$App::Prove::Plugin::TestRail::VERSION = '0.027';
 use strict;
 use warnings;
 use utf8;
@@ -12,9 +12,15 @@ use File::HomeDir qw{my_home};
 sub load {
     my ( $class, $p ) = @_;
 
-    my $app    = $p->{app_prove};
-    my $args   = $p->{'args'};
-    my $params = _parseConfig();
+    my $app  = $p->{app_prove};
+    my $args = $p->{'args'};
+
+    my $params = {};
+
+    #Only attempt parse if we aren't mocking and the homedir exists
+    my $homedir = my_home() || '.';
+    $params = TestRail::Utils::parseConfig($homedir)
+      if -e $homedir && !$ENV{'TESTRAIL_MOCKED'};
 
     my @kvp = ();
     my ( $key, $value );
@@ -34,42 +40,22 @@ sub load {
     $app->merge(1);
 
     #XXX I can't figure out for the life of me any other way to pass this data. #YOLO
-    $ENV{'TESTRAIL_APIURL'}   = $params->{apiurl};
-    $ENV{'TESTRAIL_USER'}     = $params->{user};
-    $ENV{'TESTRAIL_PASS'}     = $params->{password};
-    $ENV{'TESTRAIL_PROJ'}     = $params->{project};
-    $ENV{'TESTRAIL_RUN'}      = $params->{run};
-    $ENV{'TESTRAIL_PLAN'}     = $params->{plan};
-    $ENV{'TESTRAIL_CONFIGS'}  = $params->{configs};
-    $ENV{'TESTRAIL_VERSION'}  = $params->{version};
-    $ENV{'TESTRAIL_CASEOK'}   = $params->{case_per_ok};
-    $ENV{'TESTRAIL_STEPS'}    = $params->{step_results};
-    $ENV{'TESTRAIL_SPAWN'}    = $params->{spawn};
-    $ENV{'TESTRAIL_SECTIONS'} = $params->{sections};
+    $ENV{'TESTRAIL_APIURL'}    = $params->{apiurl};
+    $ENV{'TESTRAIL_USER'}      = $params->{user};
+    $ENV{'TESTRAIL_PASS'}      = $params->{password};
+    $ENV{'TESTRAIL_PROJ'}      = $params->{project};
+    $ENV{'TESTRAIL_RUN'}       = $params->{run};
+    $ENV{'TESTRAIL_PLAN'}      = $params->{plan};
+    $ENV{'TESTRAIL_CONFIGS'}   = $params->{configs};
+    $ENV{'TESTRAIL_VERSION'}   = $params->{version};
+    $ENV{'TESTRAIL_CASEOK'}    = $params->{case_per_ok};
+    $ENV{'TESTRAIL_STEPS'}     = $params->{step_results};
+    $ENV{'TESTRAIL_SPAWN'}     = $params->{spawn};
+    $ENV{'TESTRAIL_SECTIONS'}  = $params->{sections};
+    $ENV{'TESTRAIL_AUTOCLOSE'} = $params->{autoclose};
     return $class;
 }
 
-sub _parseConfig {
-    my $results = {};
-    my $arr     = [];
-
-    my $homedir = my_home() || '.';
-
-    open( my $fh, '<', $homedir . '/.testrailrc' )
-      or return ( undef, undef, undef );    #couldn't open!
-    while (<$fh>) {
-        chomp;
-        @$arr = split( /=/, $_ );
-        if ( scalar(@$arr) != 2 ) {
-            warn("Could not parse $_ in tlreport config\n");
-            next;
-        }
-        $results->{ lc( $arr->[0] ) } = $arr->[1];
-    }
-    close($fh);
-    return $results;
-}
-
 1;
 
 __END__
@@ -84,7 +70,7 @@ App::Prove::Plugin::TestRail - Upload your TAP results to TestRail in realtime
 
 =head1 VERSION
 
-version 0.026
+version 0.027
 
 =head1 SYNOPSIS
 
@@ -111,6 +97,7 @@ If \$HOME/.testrailrc exists, it will be parsed for any of these values in a new
     step_results=sr_sys_name
     spawn=123
     sections=section1:section2:section3: ... :sectionN
+    autoclose=0
 
 Note that passing configurations as filters for runs inside of plans are separated by colons.
 Values passed in via query string will override values in \$HOME/.testrailrc.
